permissionUrl = "https://shenmind.com/apiv2/private/upload/permission"

createPredictionUrl = 'https://shenmind.com/apiv2/private/prediction'
queryPredictionUrl = 'https://shenmind.com/apiv2/prediction/outputs'
cancelPredictionUrl = 'https://shenmind.com/apiv2/private/prediction/cancel'



import requests
import os
import time
import json
from .upload import UploadFile
from .constant import createPredictionUrl, queryPredictionUrl, cancelPredictionUrl


def run(
    modelId,
    files,
    params,
    waitResult=True
):
    data = dict()
    for key, filepath in files.items():
        if filepath.startswith('http'):
            data[key] = filepath
        else:
            storageId = UploadFile(filepath)
            data[key] = storageId
        
    # run 
    apiToken = os.environ.get('SHENMIND_API_TOKEN')
    headers = {
        'Authorization': f'Bearer {apiToken}',
        'Content-Type': 'application/json'
    }

    data['modelID'] = modelId
    data.update(params)
    response = requests.post(createPredictionUrl, data=json.dumps(data), headers=headers)

    if response.status_code == 200:
        if not waitResult:
            return response.json()['data']['predictionID']
        else:
            predictionId = response.json()['data']['predictionID']
            while True:
                prediction = getPredictionOutput(predictionId)
                if prediction['status'] in ['succeeded', 'failed', 'canceled']:
                    return prediction
                else:
                    time.sleep(1)
    else:
        raise Exception('fail to create prediction: {}'.format(response.text))

def getPredictionOutput(predictionId):
    apiToken = os.environ.get('SHENMIND_API_TOKEN')
    headers = {
        'Authorization': f'Bearer {apiToken}'
    }
    data = {
        'predictionID': predictionId
    }
    response = requests.get(queryPredictionUrl, params=data, headers=headers)

    if response.status_code == 200:
        return response.json()['data']
    else:
        raise Exception('fail to get prediction: {}'.format(response.text))


def cancelPrediction(predictionId):
    apiToken = os.environ.get('SHENMIND_API_TOKEN')
    headers = {
        'Authorization': f'Bearer {apiToken}'
    }
    data = {
        'predictionID': predictionId
    }


    response = requests.post(cancelPredictionUrl, data=data, headers=headers)


    if response.status_code == 200:
        return response.json()
    else:
        raise Exception('fail to cancel prediction: {}'.format(response.text))



import requests
import os
from .constant import permissionUrl

def get_upload_permission():
    """ Get upload permission from the server.
    
    Args:
        permission_url (str): URL to get upload permission.
        
    Returns:
        dict: Permission data.
        
    Raises:
        Exception: If failed to get permission or unexpected response format.
    """

    api_token = os.environ.get('SHENMIND_API_TOKEN')
    if not api_token:
        raise Exception("API token not available in environment variables.")


    headers = {
        'Authorization': f'Bearer {api_token}'
    }

    try:
        response = requests.get(permissionUrl, headers=headers)
        if response.status_code == 200:
            try:
                return response.json()['data']['permission']
            except KeyError:
                raise Exception("Unexpected response format: " + response.text)
        else:
            raise Exception("Failed to get upload permission: " + response.text)
        
    except requests.RequestException as e:
        raise Exception(f"Network-related error occurred: {e}")



def UploadFile(filepath):
    permission = get_upload_permission()
    
    url = permission['host']
    key = permission['directory'] + filepath.split('/')[-1]
    data = {
        'key': key,
        'policy': permission['policy'],
        'OSSAccessKeyId': permission['ossAccessKeyId'],
        'signature': permission['signature'],
        'callback': permission['callback']
    }
    
    # 准备文件数据
    with open(filepath, 'rb') as file:
        files = {'file': (filepath.split('/')[-1], file, 'application/octet-stream')}
        response = requests.post(url, data=data, files=files)

    if response.status_code == 200:
        return permission['fileID']
    else:
        raise Exception("Failed to upload file: " + response_body)




from setuptools import setup, find_packages

setup(
    name='shenmind',  # 包的名称
    version='2.0.0',  # 版本号
    author='shenmind',  # 作者姓名
    author_email='sdruc0611@gmail.com',  # 作者邮箱
    description='A Python package for shenmind',  # 包的描述信息
    packages=find_packages(),  # 包含的包
    install_requires=[  # 依赖的其他包
        'requests',
        'numpy',
    ],
    classifiers=[  # 软件分类标签
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)
